import es, playerlib, gamethread, effectlib, usermsg, random
import hosties.hosties as hosties

LR = hosties.RegisterLastRequest('redlightgreenlight')

'''
[redlightgreenlight]
en="Red Light Green Light"

[rlgl 1]
en="#lightgreen[Hosties]: $t#default has chosen to have a Red-Light Green-Light with #lightgreen$ct"

[rlgl 2]
en="#lightgreen[Hosties]: #defaultThe object of the game is to stop and go when told"

[rlgl 3]
en="#lightgreen[Hosties]: #defaultGive him/her some time to choose the end point"

[rlgl still moving]
en="#lightgreen[Hosties]: $player#default is still moving and has lost the Red-Light Green-Light"
'''

class RedLightGreenLight(object):
	def __init__(self):
		self.corners = []
		self.t = None
		self.ct = None
		self.radians = None
		self.stopped = False

	def start(self, t, ct):
		self.t = t
		self.ct = t

		hosties.HM.msg('rlgl 1', {'t': es.getplayername(t), 'ct': es.getplayername(ct)})
		hosties.HM.msg('rlgl 2')
		hosties.HM.msg('rlgl 3')

		xa, ya, za = playerlib.getPlayer(t).get('viewangle')
		xp, yp, zp = es.getplayerlocation(t)

		self.radians = ((ya - 90) / 180.0) * 3.141592654
		x_plus = 200 * math.cos(self.radians)
		y_plus = 200 * math.sin(self.radians)

		self.corners.append([xp - x_plus, yp - y_plus, zp])
		self.corners.append([xp + x_plus, yp + y_plus, zp])
		self.effectLoop()

		chooseEndLocationMenu = popuplib.easymenu('RLGL Select End Point', '_popup_choice', self.chooseEndLocationMenuSelect)
		chooseEndLocationMenu.addoption('Select Location', 'Select Location')
		popuplib.send('RLGL Select End Point', t)

	def end(self):
		self.t = None
		self.ct = None

		self.radians = None
		del self.corners[:]

		for delay in ['rlgl effect loop', 'change light', 'rlgl check won']:
			gamethread.cancelDelayed(delay)

	def effectLoop(self):
		c = self.corners
		for pair in [c[:2], c[2:]] + zip(c[:2], c[2:]):
			if not pair:
				continue
			effectlib.drawLine(pair[0], pair[1], seconds=5, red=(self.stopped) * 255, blue=0, green=(0 if self.stopped else 255))
		gamethread.delayedname(5, 'rlgl effect loop', self.effectLoop)

	def chooseEndLocationMenuSelect(self, userid, choice, popupid):
		if not userid == self.t:
			return

		x, y, z = es.getplayerlocation(userid)
		x_plus = 200 * math.cos(self.radians)
		y_plus = 200 * math.sin(self.radians)

		self.corners.append([x - x_plus, y - y_plus, z])
		self.corners.append([x + x_plus, y + y_plus, z])

		left = random.choice([self.t, self.ct])
		right = self.t if left == self.ct else self.ct

		x_plus *= 0.5
		y_plus *= 0.5

		es.server.queuecmd('es_xsetpos %s %s %s %s'%(left, x - x_plus, y - y_plus, z + 10))
		es.server.queuecmd('es_xsetpos %s %s %s %s'%(right, x + x_plus, y + y_plus, z + 10))

		leftPlayer = playerlib.getPlayer(left)
		rightPlayer = playerlib.getPlayer(right)

		leftPlayer.set('freeze', 1)
		rightPlayer.set('freeze', 1)

		for delay, message in enumerate(['3', '2', '1', 'Go!']):
			gamethread.delayed(delay, es.centermsg, message)

		gamethread.delayed(3, leftPlayer.set, ('noclip', 0))
		gamethread.delayed(3, rightPlayer.set, ('noclip', 0))

		self.stopped = 0
		gamethread.delayedname(random.randint(40, 70) / 10.0, 'change light', self.changeLightLoop)
		self.checkWonLoop()

	def changeLightLoop(self):
		self.stopped = 0 if self.stopped else 1
		plus = 6 if self.stopped else 0

		gamethread.delayedname(random.randint(10 + plus, 40 + plus) / 10.0, 'change light', self.changeLightLoop)
		if self.stopped:
			gamethread.delayed(0.5, self.checkMoving)

	def checkMoving(self):
		moving = []
		for userid in [self.t, self.ct]:
			for v in es.getplayermovement(userid):
				if v:
					hosties.HM.msg('rlgl still moving', {'player': es.getplayername(userid)})
					moving.append(userid)
					break

		if len(moving) == 2:
			hosties.getPlayer(self.t).makeRebel()
			return

		if moving:
			LR.stopLR(self.t, self.t if moving[0] == self.ct else self.ct)

	def checkWonLoop(self):
		gamethread.delayedname(0.1, 'rlgl check won', self.checkWonLoop)

RLGL = RedLightGreenLight()

LR.registerStartFunction(RLGL.start)
LR.registerEndFunction(RLGL.end)
